package ru.redguy.dynamicshop.api;

import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.YamlConfiguration;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import ru.redguy.dynamicshop.DynamicShop;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;

public class Database {
    private static final Database instance = new Database();
    private final File file;
    private final YamlConfiguration config;
    private static final String ITEMS_KEY = "items";

    private Database() {
        this.file = new File(DynamicShop.getInstance().getDataFolder(), "items.yml");
        this.config = new YamlConfiguration();
        loadDatabase();
    }

    @NotNull
    public static Database getInstance() {
        return instance;
    }

    private synchronized void loadDatabase() {
        try {
            if (!file.exists()) {
                DynamicShop.getInstance().getDataFolder().mkdirs();
                file.createNewFile();
            }
            config.load(file);
        } catch (IOException | InvalidConfigurationException e) {
            DynamicShop.getInstance().getLogger().log(Level.SEVERE, "Failed to load database", e);
            throw new RuntimeException("Failed to load database", e);
        }
    }

    private synchronized void saveDatabase() {
        try {
            config.save(file);
        } catch (IOException e) {
            DynamicShop.getInstance().getLogger().log(Level.SEVERE, "Failed to save database", e);
            throw new RuntimeException("Failed to save database", e);
        }
    }

    @NotNull
    public synchronized List<ShopItem> getItems() {
        List<?> list = config.getList(ITEMS_KEY);
        if (list == null) {
            return new ArrayList<>();
        }

        List<ShopItem> shopItems = new ArrayList<>();
        for (Object obj : list) {
            if (obj instanceof ShopItem) {
                shopItems.add((ShopItem) obj);
            } else {
                DynamicShop.getInstance().getLogger().warning("Found invalid item in database: " + obj);
            }
        }
        return Collections.unmodifiableList(shopItems);
    }

    public synchronized void setItems(@NotNull List<ShopItem> items) {
        if (items == null) {
            throw new IllegalArgumentException("Items list cannot be null");
        }
        config.set(ITEMS_KEY, new ArrayList<>(items)); // Create a copy to prevent external modification
        saveDatabase();
    }

    @Nullable
    public synchronized ShopItem getItem(int index) {
        List<ShopItem> items = getItems();
        if (index < 0 || index >= items.size()) {
            return null;
        }
        return items.get(index).clone(); // Return a clone to prevent modification
    }

    public synchronized void setItem(int index, @NotNull ShopItem item) {
        if (item == null) {
            throw new IllegalArgumentException("Item cannot be null");
        }
        List<ShopItem> items = new ArrayList<>(getItems());
        if (index < 0) {
            throw new IllegalArgumentException("Index cannot be negative");
        }
        if (index >= items.size()) {
            throw new IllegalArgumentException("Index " + index + " is out of bounds for size " + items.size());
        }
        items.set(index, item);
        setItems(items);
    }

    public synchronized void addItem(@NotNull ShopItem item) {
        if (item == null) {
            throw new IllegalArgumentException("Item cannot be null");
        }
        List<ShopItem> items = new ArrayList<>(getItems());
        items.add(item);
        setItems(items);
    }

    public synchronized void removeItem(int index) {
        List<ShopItem> items = new ArrayList<>(getItems());
        if (index < 0 || index >= items.size()) {
            throw new IllegalArgumentException("Index " + index + " is out of bounds for size " + items.size());
        }
        items.remove(index);
        setItems(items);
    }

    public synchronized void clear() {
        setItems(new ArrayList<>());
    }

    public synchronized int size() {
        return getItems().size();
    }
}
