package ru.redguy.dynamicshop.api;

import org.bukkit.Material;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@SerializableAs("ShopItem")
public class ShopItem implements Cloneable, ConfigurationSerializable {
    private final ItemStack item;
    private final double price;

    /**
     * Creates a new shop item
     *
     * @param item The item to sell
     * @param price The price of the item
     * @throws IllegalArgumentException if item is null or price is negative
     */
    public ShopItem(@NotNull ItemStack item, double price) {
        if (item == null || item.getType() == Material.AIR) {
            throw new IllegalArgumentException("Item cannot be null or AIR");
        }
        if (price < 0) {
            throw new IllegalArgumentException("Price cannot be negative");
        }
        this.item = item.clone(); // Defensive copy
        this.price = price;
    }

    /**
     * Gets the item being sold
     *
     * @return A copy of the item
     */
    @NotNull
    public ItemStack getItem() {
        return item.clone(); // Return defensive copy
    }

    /**
     * Gets the price of the item
     *
     * @return The price
     */
    public double getPrice() {
        return price;
    }

    @Contract("_ -> new")
    public static @NotNull ShopItem deserialize(@NotNull Map<String, Object> args) {
        ItemStack item = args.containsKey("item") ? (ItemStack) args.get("item") : new ItemStack(Material.AIR);
        double price = args.containsKey("price") ? (Double) args.get("price") : 0.0;
        
        return new ShopItem(item, price);
    }

    @Override
    public @NotNull Map<String, Object> serialize() {
        Map<String, Object> map = new HashMap<>();
        map.put("item", item);
        map.put("price", price);
        return map;
    }

    @Override
    public ShopItem clone() {
        try {
            return new ShopItem(this.item.clone(), this.price);
        } catch (Exception e) {
            throw new AssertionError("Failed to clone ShopItem: " + e.getMessage());
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof ShopItem)) return false;
        ShopItem shopItem = (ShopItem) o;
        return Double.compare(shopItem.price, price) == 0 &&
                Objects.equals(item, shopItem.item);
    }

    @Override
    public int hashCode() {
        return Objects.hash(item, price);
    }

    @Override
    public String toString() {
        return "ShopItem{" +
                "item=" + item +
                ", price=" + price +
                '}';
    }
}
